package com.blazinggames.ootw.ep33
{
	// Adobe Imports
	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.MouseEvent;
//	import flash.text.TextField;
//	import flash.utils.getTimer;
		
	/**
	 * The cube used in the puzzle.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007 Blazing Games Inc. All Rights Reserved.
	*/
	public class Cube
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		// side constants
		public static const SIDE_TOP:int = 0;
		public static const SIDE_FRONT:int = 1;
		public static const SIDE_LEFT:int = 2;
		public static const SIDE_BOTTOM:int = 3;
		public static const SIDE_BACK:int = 4;
		public static const SIDE_RIGHT:int = 5;

		// states
		public static const STATE_NORMAL:int = 0;
		public static const STATE_SWAP:int = 1;
		public static const STATE_SELECTED:int = 2;

		// events
		public static const SELECTED_CUBE_EVENT:String = "SELECTED_CUBE";
		public static const SWAPPED_CUBE_EVENT:String = "SWAPPED_CUBE";
		public static const DESELECTED_CUBE_EVENT:String = "DESELECTED_CUBE";

// *********************
// ***** VARIABLES *****
// *********************

		protected var _curState:int;
		protected var _cubeID:int;
		protected var _faces:Array;
		
		// from movie clip:
		// _select_btn
		// _swap_btn
		// _faceFront_movie
		// _faceTop_movie
		// _faceLeft_movie
		// _rotateDown_btn
		// _rotateRight_btn
		
// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function Cube()
		{
			_faces = new Array(1,2,3,4,5,6);
			updateFaceDisplay();
			_select_btn.addEventListener(MouseEvent.CLICK, handleSelect);
			_swap_btn.addEventListener(MouseEvent.CLICK, handleSwap);
			_rotateDown_btn.addEventListener(MouseEvent.CLICK,handleRotateDown);
			_rotateRight_btn.addEventListener(MouseEvent.CLICK, handleRotateRight);
			setState(0);
		}

// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handleSelect(evt:Event)
		{
			var s:String;
			
			if (_curState > 0) {
				s = DESELECTED_CUBE_EVENT;
				setState(0);
			} else {
				s =  SELECTED_CUBE_EVENT;
				setState(2);
			}
			var evnt:Event = new Event(s);
			this.dispatchEvent(evnt);
		}

		public function handleSwap(evt:Event)
		{
			setState(0);
			var evnt:Event = new Event(SWAPPED_CUBE_EVENT);
			this.dispatchEvent(evnt);
		}

		public function handleRotateDown(evt:Event)
		{
			var temp:int = _faces[SIDE_TOP];
			_faces[SIDE_TOP] = _faces[SIDE_BACK];
			_faces[SIDE_BACK] = _faces[SIDE_BOTTOM];
			_faces[SIDE_BOTTOM] = _faces[SIDE_FRONT];
			_faces[SIDE_FRONT] = temp;
			updateFaceDisplay();
		}

		public function handleRotateRight(evt:Event)
		{
			var temp:int = _faces[SIDE_FRONT];
			_faces[SIDE_FRONT] = _faces[SIDE_LEFT];
			_faces[SIDE_LEFT] = _faces[SIDE_BACK];
			_faces[SIDE_BACK] = _faces[SIDE_RIGHT];
			_faces[SIDE_RIGHT] = temp;
			updateFaceDisplay();
		}

// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function setState(cubeState:int)
		{
			_curState = cubeState;
			
			switch (cubeState) {
				case STATE_NORMAL: 
					_select_btn.visible = true;
					_swap_btn.visible = false;
					_faceTop_movie.visible = false;
					_faceLeft_movie.visible = false;
					_rotateDown_btn.visible = false;
					_rotateRight_btn.visible = false;
					gotoAndStop("Normal");
					break;
				case STATE_SWAP:
					_select_btn.visible = false;
					_swap_btn.visible = true;
					_faceTop_movie.visible = false;
					_faceLeft_movie.visible = false;
					_rotateDown_btn.visible = false;
					_rotateRight_btn.visible = false;
					gotoAndStop("Swap");
					break;
				case STATE_SELECTED:
					_select_btn.visible = true;
					_swap_btn.visible = false;
					_faceTop_movie.visible = true;
					_faceLeft_movie.visible = true;
					_rotateDown_btn.visible = true;
					_rotateRight_btn.visible = true;
					gotoAndStop("Selected");
					break;
			}
		}

		public function setID(id:int)
		{
			_cubeID = id; 
		}
		
		public function getID():int
		{
			return _cubeID; 
		}
		
		public function setFaceTile(face:int, tileID:int)
		{
			_faces[face] = tileID;
			updateFaceDisplay();
		}
		
		public function getFaceTile(face:int):int
		{
			return _faces[face];
		}
		
		protected function updateFaceDisplay()
		{
			_faceFront_movie.gotoAndStop(_faces[SIDE_FRONT]);
			_faceTop_movie.gotoAndStop(_faces[SIDE_TOP]);
			_faceLeft_movie.gotoAndStop(_faces[SIDE_LEFT]);
		}
	} // end class
} // end package

