package com.blazinggames.ootw.ep33
{
	// Adobe Imports
//	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.EventDispatcher;
//	import flash.events.MouseEvent;
//	import flash.text.TextField;
//	import flash.utils.getTimer;
		
	/**
	 * The cube puzzle.
	 *
	 * @author Billy D. Spelchan
	 * @version 1.0
	 * Copyright (c) 2007 Blazing Games Inc. All Rights Reserved.
	*/
	public class CubePuzzleBoard
	extends MovieClip
	{
// *********************
// ***** CONSTANTS *****
// *********************

		public static const PUZZLE_SOLVED_EVENT:String = "PUZZLE_SOLVED";

// *********************
// ***** VARIABLES *****
// *********************

		protected var _lastSelected:int;
		protected var _cubes:Array;

		protected var _puzzleData:Array;

// ***********************
// ***** CONSTRUCTOR *****
// ***********************

		public function CubePuzzleBoard()
		{
			_cubes = new Array(
					_cubeTL_movie, _cubeTC_movie, _cubeTR_movie,
					_cubeML_movie, _cubeMC_movie, _cubeMR_movie,
					_cubeBL_movie, _cubeBC_movie, _cubeBR_movie );
					
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				_cubes[cntr].setID(cntr);
				_cubes[cntr].addEventListener(Cube.SELECTED_CUBE_EVENT, 
						handleSelect);
				_cubes[cntr].addEventListener(Cube.SWAPPED_CUBE_EVENT, 
						handleSwap);
				_cubes[cntr].addEventListener(Cube.DESELECTED_CUBE_EVENT, 
						handleDeselect);
			}
			
			_puzzleData = new Array(
				new Array( 1, 2, 3, 4, 5, 6, 7, 8, 9),
				new Array(10,11,12,13,14,15,16,17,18),
				new Array(19,20,21,22,23,24,25,26,27),
				new Array(28,29,30,31,32,33,34,35,36),
				new Array(37,38,39,40,41,42,43,44,45),
				new Array(46,47,48,49,50,51,52,53,54) );
			createPuzzle();
		}

// **************************************
// ***** Functions - Event handlers *****
// **************************************

		public function handleSelect(evt:Event)
		{
			_lastSelected = evt.currentTarget.getID();
			trace ("Cube " + _lastSelected + " selected");
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				if (cntr != _lastSelected)
					_cubes[cntr].setState(Cube.STATE_SWAP);
			}
		}

		public function handleSwap(evt:Event)
		{
			var selected:int = evt.currentTarget.getID();
			var cntr:int, temp:int;
			
			trace ("swapping " + _lastSelected + " and " + selected);
			for (cntr = 0; cntr < 6; ++cntr) {
				temp = _cubes[_lastSelected].getFaceTile(cntr);
				_cubes[_lastSelected].setFaceTile(cntr, _cubes[selected].getFaceTile(cntr));
				_cubes[selected].setFaceTile(cntr, temp);
			}
			for (cntr = 0; cntr < 9; ++cntr) {
				_cubes[cntr].setState(Cube.STATE_NORMAL);
			}
			checkIfWon();
		}

		public function handleDeselect(evt:Event)
		{
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				_cubes[cntr].setState(Cube.STATE_NORMAL);
			}
			checkIfWon();
		}

		protected function checkIfWon():Boolean
		{
			var matches:int = 0;
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				if (_cubes[cntr].getFaceTile(Cube.SIDE_FRONT) == (cntr+1))
					++matches;
			}

			if (matches == 9) {
				var evnt:Event = new Event(PUZZLE_SOLVED_EVENT);
				this.dispatchEvent(evnt);
				return true;
			} else
				return false;
		}
		
		public function countCorrectFaces():int
		{
			var matches:int = 0;
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				if (_cubes[cntr].getFaceTile(Cube.SIDE_FRONT) < 10)
					++matches;
			}
			return matches
		}
		
		public function countProperlyPlaced():int
		{
			var matches:int = 0;
			for (var cntr:int = 0; cntr < 9; ++cntr) {
				if (_cubes[cntr].getFaceTile(Cube.SIDE_FRONT) == (cntr+1))
					++matches;
			}
			return matches
		}
/*
		public function textUpdate(evt:Event)
		{
			var evnt:Event = new Event(STRING_FINISHED_EVENT);
			this.dispatchEvent(evnt);
		}
*/
// ***************************************
// ***** Functions - General control *****
// ***************************************

		public function createPuzzle()
		{
			var cntrCube:int, cntrSide:int, temp:int, swapIndex:int;

			// first reset the puzzle so the puzzle we create will be valid
			for (cntrSide = 0; cntrSide < 6; ++cntrSide)
				for (cntrCube = 0; cntrCube < 9; ++cntrCube)
					_puzzleData[cntrSide][cntrCube] = cntrSide*9 + cntrCube + 1;

			// next scrable the six pictures
			for (cntrSide = 0; cntrSide < 6; ++cntrSide) {
				for (cntrCube = 0; cntrCube < 9; ++cntrCube) {
					swapIndex = Math.floor(Math.random()*9);
					temp = _puzzleData[cntrSide][swapIndex];
					_puzzleData[cntrSide][swapIndex] = 
							_puzzleData[cntrSide][cntrCube];
					_puzzleData[cntrSide][cntrCube] = temp;
				}
			}

			// now scramble each of the cubes
			for (cntrCube = 0; cntrCube < 9; ++cntrCube) {
				for (cntrSide = 0; cntrSide < 6; ++cntrSide) {
					swapIndex = Math.floor(Math.random()*6);
					temp = _puzzleData[swapIndex][cntrCube];
					_puzzleData[swapIndex][cntrCube] = 
							_puzzleData[cntrSide][cntrCube];
					_puzzleData[cntrSide][cntrCube] = temp;
				}
			}

			// finally set the faces of the nine cubes
			for (cntrCube = 0; cntrCube < 9; ++cntrCube) {
				for (cntrSide = 0; cntrSide < 6; ++cntrSide) {
					_cubes[cntrCube].setFaceTile(cntrSide, 
							_puzzleData[cntrSide][cntrCube]);
				}
			}
		}

	} // end class
} // end package

